<?php
namespace Teknisa\CDP\Controllers;

use Teknisa\CDP\Controllers\TeknisaCrud;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\ParameterBag;
use \Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Teknisa\CDP\Services\AtividadeAppService;

class AtividadeAppController extends TeknisaCrud { 

    protected $service; 
    protected $parameterBag;
    protected $entityManager;
    
    public function __construct($crudService, ParameterBag $parameterBag, EntityManager $entityManager, AtividadeAppService $service)
    {        
        $this->service = $service;
        $this->entityManager = $entityManager;
        $this->AtividadeAppService = $service;
        parent::__construct($crudService, $parameterBag);
    }
    
   
    public function start_atividade(DTO\Request $request, DTO\Response $response){
        try {   
            
            // start atividade
            $row  = $request->getRow(); 
            $result = $this->service->start($row);
            
            $response->addDataSet(new DataSet('/start_atividade', [$result]));
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function stop_atividade(DTO\Request $request, DTO\Response $response){
        try {   
            
            $row  = $request->getRow();
            $result = $this->service->stop($row);
            
            $response->addDataSet(new DataSet('/stop_atividade', [$result]));
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function pause_atividade(DTO\Request $request, DTO\Response $response){
        try {   
            
            $row  = $request->getRow();
            $result = $result = $this->service->pause($row);
            
            $response->addDataSet(new DataSet('/pause_atividade', [$result]));
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function unpause_atividade(DTO\Request $request, DTO\Response $response){
        try {   
            
            $row  = $request->getRow();
            $result = $this->service->unpause($row);
            
            $response->addDataSet(new DataSet('/unpause_atividade', [$result]));
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function searchChild(DTO\Request $request, DTO\Response $response){
        try {   
            
            $row  = $request->getRow();
            $result = $this->service->searchChild($row);
            
            $response->addDataSet(new DataSet('/searchChild', [$result]));
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function searchLastCompleteChild(DTO\Request $request, DTO\Response $response){
        try {   
            
            $row  = $request->getRow();
            $result = $this->service->searchLastCompleteChild($row);
            
            $response->addDataSet(new DataSet('/searchLastCompleteChild', [$result]));
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function sendJustificative(DTO\Request $request, DTO\Response $response){
        try {   
            
            $row  = $request->getRow();
            $result = $this->service->sendJustificative($row);
            
            $response->addDataSet(new DataSet('/sendJustificative', [$result]));
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function sendObservacao(DTO\Request $request, DTO\Response $response){
        try {   
            
            $row  = $request->getRow();
            $result = $this->service->sendObservacao($row);
            
            $response->addDataSet(new DataSet('/sendObservacao', [$result]));
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function getDescricaoHtml(DTO\Request $request, DTO\Response $response) {
        
        $row  = $request->getRow();
        
        $result  = $this->service->rtf2Html($row);
        
        $response->addDataSet(new DataSet('/getDescricaoHtml', [$result]));
        
        /*$dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
        $this->dataSourceName = $dataSource;
        parent::find($request, $response);
        
        $data  = $response->getDataSets();
        $rows = $data[0]->getRows();
        
        $rows  = $this->service->rtf2Html($rows);
        
        $dataSet = new DataSet("observclie_operador", $rows);
        $response->setDataSets(array($dataSet));*/
    }
        
}