<?php
namespace Teknisa\CDP\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode; 
use Teknisa\CDP\Factories\AtividadeAppFactory;

class AtividadeAppService {
    
    protected $entityManager;
    protected $environment;
    protected $newCode;
    protected $AtividadeAppFactory;
    
    
    public function __construct(EntityManager $entityManager, Environment $environment, NewCode $newCode, $AtividadeAppFactory)
    {        
        $this->entityManager                       = $entityManager;
        $this->environment                         = $environment;
        $this->newCode                             = $newCode;
        $this->AtividadeAppFactory                  = $AtividadeAppFactory;        
    }
    
    public function start($row){   
        try {    
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $isCompleted = $this->AtividadeAppFactory->isCompleted($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['IDSITGRAVCONT'], $row['NRORG']);
            if(empty($isCompleted)){
                
                $inProgress = $this->AtividadeAppFactory->inProgress($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['NRORG']);
                $isPending = $this->AtividadeAppFactory->isPending($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['IDSITGRAVCONT'], $row['NRORG']);
                
                if (empty($isPending) || empty($inProgress)) {
                    /** Checks if the activity was the first to be initialized */
                    $isTheFirst = $this->AtividadeAppFactory->isTheFirst($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRORG']);
                    
                    if ($isTheFirst[0]['COUNT'] == 0) {
                        $this->AtividadeAppFactory->initTask($row['STARTDATE'], $row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['CDOPERADOR']);
                        $this->AtividadeAppFactory->changeStatusAudit($row['CDFILIAL'], $row['NRAUDITORIA'], '2');
                    }
                    
                    $return = $this->AtividadeAppFactory->initActivity($row['STARTDATE'], $row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['NRORG'], $row['CDOPERADOR']);
                    
                    $this->entityManager->commit();
                    return $return;
                }
                else {
                   /** Is already in progress. The activity cannot be initialized.*/
                   throw new \Exception('Atividide já esta em progresso.');
                }
            } else {
                throw new \Exception('Atividide já foi concluida.');
            }
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function stop($row){
        try {    
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $isCompleted = $this->AtividadeAppFactory->isCompleted($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['IDSITGRAVCONT'], $row['NRORG']);
            if(empty($isCompleted)){
                
                $inProgress = $this->AtividadeAppFactory->inProgress($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['NRORG']);
                $isPending = $this->AtividadeAppFactory->isPending($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['IDSITGRAVCONT'], $row['NRORG']);
                
                if($inProgress){
                    /** Checks if the activity is the last*/
                    
                    $isTheLast = $this->AtividadeAppFactory->isTheLast($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRORG']);
                    
                    if ($isTheLast[0]['COUNT'] == 1) {
                        $dtIni = $this->AtividadeAppFactory->getTaskDtInit($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI']);
                        $this->AtividadeAppFactory->completeTask($dtIni[0]['DTINIRINSAUD'], $row['END_DATE'], $row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP']);
                        $this->AtividadeAppFactory->changeStatusAudit($row['CDFILIAL'], $row['NRAUDITORIA'], '4');
                    }
                    if($isPending){
                       $maxNRRECUETAPOP = $this->AtividadeAppFactory->SELECTmaxNRRECUETAPOPComplete($row);
                       $searchChild = $this->AtividadeAppFactory->searchChild($row);
                       if($searchChild){
                           $qttotal = $this->tempoTotal($row["TMREAL"]);
                           $status = $this->AtividadeAppFactory->completeAndPauseActivity($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['NRORG'], $qttotal);
                           $row['PAUSE_DATE'] = $row['END_DATE'];
                           $row['NRRECUETAPOP'] = $maxNRRECUETAPOP; 
                           $this->AtividadeAppFactory->pauseActivity($row);
                           $this->entityManager->commit();
                           return $status;
                       }else{
                           $qttotal = $this->tempoTotal($row["TMREAL"]);
                           $status = $this->AtividadeAppFactory->completeActivity($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['NRORG'], $qttotal, $row['DTFINUTRECOP']);
                           $row['PAUSE_DATE'] = $row['END_DATE'];
                           $row['NRRECUETAPOP'] = $maxNRRECUETAPOP; 
                           $this->entityManager->commit();
                           return $status;
                       }
                    }else{
                       $qttotal = $this->tempoTotal($row["TMREAL"]);
                       $status = $this->AtividadeAppFactory->completePausedActivity($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['NRORG'], $qttotal);
                       $this->entityManager->commit();
                       return $status; 
                    }
                } else {
                    /** isn't in progress*/
                    throw new \Exception('Atividide não está em andamento.');
                }
            } else {
                throw new \Exception('Atividide já foi concluida.');
            }
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function tempoTotal ($time){
        $hora    = substr($time, 0, 2);
        $horaEmSeg = intval($hora) * 3600;
        $minuto  = substr($time, 3, -3);
        $minutoEmSeg = intval($minuto) * 60;
        $segundo = substr($time, 6, 7);
        $totalSeundos = $horaEmSeg + $minutoEmSeg + $segundo;
        $totalMinutos = $totalSeundos / 60;
        $totalHora = $totalMinutos / 60;
        
        return $totalHora;
    }
    
    public function pause($row){
        try {    
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $isCompleted = $this->AtividadeAppFactory->isCompleted($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['IDSITGRAVCONT'], $row['NRORG']);
            if(empty($isCompleted)){
                
                $inProgress = $this->AtividadeAppFactory->inProgress($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['NRORG']);
                //$isPending = $this->AtividadeAppFactory->isPending($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['IDSITGRAVCONT'], $row['NRORG']);
                
                if($inProgress){
                    $searchChild = $this->AtividadeAppFactory->searchChild($row);
                    if($searchChild){
                        $pauseActivityFather = $this->AtividadeAppFactory->pauseActivityFather($row);
                        if($pauseActivityFather){
                            $row['NRRECUETAPOP'] = $searchChild[0]['NRRECUETAPOP'];
                            $status = $this->AtividadeAppFactory->pauseActivity($row);
                            $this->entityManager->commit();
                            return $status;
                        } else {
                            throw new \Exception('Erro ao atualizar o registro pai.');
                        }
                    } else {
                        $status = $this->AtividadeAppFactory->pauseActivity($row);
                        $this->entityManager->commit();
                        return $status;
                    }
                } else {
                    /** isn't in progress*/
                    throw new \Exception('Atividide não está em andamento.');
                }
            } else {
                throw new \Exception('Atividide já foi concluida.');
            }
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function unpause($row){
        try {    
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $isCompleted = $this->AtividadeAppFactory->isCompleted($row['CDFILIAL'], $row['NRAUDITORIA'], $row['NRINSPECAUDI'], $row['NRRECUETAPOP'], $row['IDSITGRAVCONT'], $row['NRORG']);
            if(empty($isCompleted)){
                
                    $status = $this->AtividadeAppFactory->unpauseActivity($row);
                    $this->entityManager->commit();
                    return $status;
        
            } else {
                throw new \Exception('Atividide já foi concluida.');
            }
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function searchChild($row){
        try {    
             
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $searchChild = $this->AtividadeAppFactory->searchChild($row);
            if(empty($searchChild) && (!$row['PAUSED_STATUS'])){
                throw new \Exception('Ocorreu um erro.');
            } else {
                    $this->entityManager->commit();
                    return $searchChild;
            }
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function searchLastCompleteChild($row){
        try {    
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $searchLastCompleteChild = $this->AtividadeAppFactory->searchLastCompleteChild($row);
            $this->entityManager->commit();
            return $searchLastCompleteChild;
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function sendJustificative($row){
        try {    
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $lastID = $this->AtividadeAppFactory->lastIDJustificapausa($row);
            $row['CDJUSTIFICA'] = $lastID[0]['LASTID'];
            
            $sendJustificative = $this->AtividadeAppFactory->sendJustificative($row);
            
            $this->entityManager->commit();
            return $sendJustificative;
            
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function sendObservacao($row){
        try {    
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $existeNaTabela = $this->AtividadeAppFactory->existeNaTabela($row);
            if($existeNaTabela){
                $sendObservacao = $this->AtividadeAppFactory->sendObservacaoUpdate($row);
            } else {
                $sendObservacao = $this->AtividadeAppFactory->sendObservacaoInsert($row);
            }
            $this->entityManager->commit();
            return $sendObservacao;
            
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function rtf2Html ($rows){
        try {

            $obs = $rows['TXINSPECAUDIhtml'];
            if(substr($obs, 0, 5) == '{\rtf'){
                $note = new \JoshRibakoff_Note;
                $note->setRTF($obs);
                $html = $note->formatHTML();

                $html = substr($html, 0, -6);
                
                // $html = $this->setTamanhoHtml($html, $obs);
                
                return $html;
            }else return $obs;

        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /*
        A biblioteca não realiza a conversão de maneira adequada, sendo que algumas 
        formatações ficam perdidas.Então tentamos fazer que o tamanho
        da fonte seja mantido durante a conversão;
    */
    public function setTamanhoHtml ($html, $rtf){
        $tamanhofontes = $this->retornarTamnhosFonte($rtf);
        // caso o html possua a tag span
        if(strstr($html, "<span")){
            if(!empty($tamanhofontes)){
                $textoHtml     = '';
                $ultimoTamanho = end($tamanhofontes);
                $tagsSpan  = $this->retornaTagSpan($html);
                for($i=0; $i< count($tagsSpan); $i++){
                    // Verifica se o tamanho corresponde a tag span encontra no array dos tamnahos.Se não pega o utlimo tamanho.
                    $tamanho    = isset($tamanhofontes[$i+1]) ? $tamanhofontes[$i] : $ultimoTamanho;
                    
                    $pos        = strpos($tagsSpan[$i], 'style="');
                    $textoHtml .= substr($tagsSpan[$i],0,$pos+7);
                    $textoHtml .= 'font-size:'.$tamanho.'px;';
                    $textoHtml .= substr($tagsSpan[$i],$pos+7);
                }
                return $textoHtml;
            }else{
                return $html;
            }
        }else{
            return '<span style ="font-size:'.$tamanhofontes[0].'px">'.$html.'</span>';
            
        }
    }
    
    /* Função que percorre o texto rtf, buscnado todos os tamnhos de fontes que existem nele.
    O token responsavel pelo tamanho da fonte é \fs. */
    public function retornarTamnhosFonte ($rtf){
        $tamanhosFontes     = array();
        $numero_ocorrencias = 0;
        while( ($numero_ocorrencias = strpos($rtf, '\fs', $numero_ocorrencias+1) ) != 0)
        {
            $string       = substr($rtf,$numero_ocorrencias+3);
            $terminoFonte = strpos($string, " ");
            
            //Dividi por 2 para encontrar o tamanho real em html
            array_push($tamanhosFontes, strval (floatval(substr($string,0,$terminoFonte))/2)) ;
        }
        return $tamanhosFontes;
        
    }
    
    //Retorna todas as tags span presente na HTML
    public function retornaTagSpan ($html){
        $tagsSpan = array();
        $abreTag  = array();
        $fechaTag = array();
        $offset   = 0;
        
        while ( ($pos = strpos($html, '<span', $offset)) !== false) {
            
            $abreTag [] = $pos;
            $offset     = $pos+1;
        }
        $offset = 0;
        while ( ($pos = strpos($html, '</span>', $offset)) !== false) {
            
            $fechaTag [] = $pos;
            $offset      = $pos+1;
        }
        
        for($i=0; $i<count($abreTag); $i++){
            $tagsSpan [] = substr ($html,$abreTag[$i], $fechaTag[$i]+7);
        }
        return $tagsSpan; 
    }
    
}