<?php
/**
 * Geral\Repositories
 *
 * @category Php
 * @package  Geral\Repositories
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Geral\Repositories;

use Geral\Util\TratamentoDeMensagens;

/**
 * Class Fornecedor
 *
 * @category Php
 * @package  Geral\Repositories
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class Fornecedor extends \Doctrine\ORM\EntityRepository
{
    /**
     * Consulta Fornecedor Vinculado ao Produto
     * @param String $nrorg      Nrorg
     * @param String $cdforneced Fornecedor
     * @param String $cdproduto  Produto
     * @return array
     * @throws \Exception
     */
    public function consultaFornecedorVinculadoProduto(
        $nrorg, $cdforneced, $cdproduto
    ) {
        $sql = "SELECT COUNT(*) as CONTA
                FROM PRODFORN, PRODUTO
                WHERE
                   PRODFORN.NRORG     = PRODUTO.NRORG       AND
                   PRODFORN.CDPRODUTO = PRODUTO.CDPRODUTO   AND
                   :P_CDPRODUTO LIKE PRODUTO.CDARVPROD||'%' AND
                   PRODFORN.NRORG      = :P_NRORG           AND
                   PRODFORN.CDFORNECED = :P_CDFORNECED";

        $paramsWhere = array(
            'P_NRORG'      => $nrorg,
            'P_CDFORNECED' => $cdforneced,
            'P_CDPRODUTO'  => $cdproduto,
        );

        try {
            return $this->_em->getConnection()->fetchAll($sql, $paramsWhere);
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Verifica o status e qualificacao do fornecedor.
     *
     * @param String $cdforneced Fornecedor
     * @param String $cdfilial   Filial
     * @param String $dtautoforn DtAutoFornecedor
     * @param String $nrorg      Nrorg
     *
     * @throws \Exception
     * @return mixed
     */
    public function getStatusDeQualificacaoFornecedorImportacaoAF(
        $cdforneced, $cdfilial, $dtautoforn, $nrorg
    ) {
        $sql = "
          SELECT FORNECEDOR.CDFORNECED
            FROM FORNECEDOR, QUALIFICACAO, PARASUPRI
            WHERE FORNECEDOR.CDQUALIF IS NOT NULL
              AND FORNECEDOR.CDFORNECED      = :CDFORNECED
              AND FORNECEDOR.CDQUALIF        = QUALIFICACAO.CDQUALIF
              AND FORNECEDOR.NRORG           = QUALIFICACAO.NRORG
              AND PARASUPRI.CDFILIAL         = :CDFILIAL
              AND PARASUPRI.NRORG            = :NRORG
              AND (
                QUALIFICACAO.IDPERCOTCMP     = '1'
                OR (QUALIFICACAO.IDPERCOTCMP = '2' AND FORNECEDOR.IDFORNAUTO = 'S')
              )
              AND (
                (PARASUPRI.IDULTIDTVALQ    = 'D'  AND TO_DATE(:DTAUTOFORN) <= FORNECEDOR.DTVALQUALIF)
                OR (PARASUPRI.IDULTIDTVALQ = 'M' AND TO_DATE(:DTAUTOFORN) <= LAST_DAY(FORNECEDOR.DTVALQUALIF))
              )

          UNION

          SELECT FORNECEDOR.CDFORNECED
            FROM FORNECEDOR
            WHERE FORNECEDOR.CDFORNECED = :CDFORNECED
              AND FORNECEDOR.CDQUALIF IS NULL OR FORNECEDOR.IDPERCMPDFE = 'S'
              AND FORNECEDOR.NRORG = :NRORG
        ";

        $params = array(
            "CDFORNECED" => $cdforneced,
            "CDFILIAL"   => $cdfilial,
            "DTAUTOFORN" => $dtautoforn,
            "NRORG"      => $nrorg,
        );
        try {
            return $this->getEntityManager()->getConnection()->fetchColumn($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica Qualificação por Fornecedor
     *
     * @param String $nrorg      Organização
     * @param String $cdprodini  Produto Inicial
     * @param String $cdprodfin  Produto Final
     * @param String $tproduto   Todos Produtos
     * @param String $cdforneced Fornecedor
     * @param String $tforn      Todos Fornecedores
     * @param String $cdfilial   Filial
     * @param String $dtcol      Data da Coleta
     *
     * @throws \Exception
     * @return array
     */
    public function verificaQualificacaoFornecedor(
        $nrorg, $cdprodini, $cdprodfin, $tproduto,
        $cdforneced, $tforn, $cdfilial, $dtcol
    ) {
        $sql = "SELECT
                    FORNECEDOR.CDFORNECED
                FROM
                    FORNECEDOR,
                    PRODUTO
                WHERE
                    ((PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN)
                    OR :P_TPRODUTO = 'T')
                    AND (PRODUTO.CDPRODESTO IS NOT NULL)
                    AND (INSTR(:P_CDFORNECED, FORNECEDOR.CDFORNECED) <> 0
                    OR :P_TFORN = 'T')
                    AND NOT EXISTS
                        (SELECT QFOPRODFORN.CDPRODUTO
                        FROM
                            QFOPRODFORN,
                            PARASUPRI
                        WHERE
                            PARASUPRI.CDFILIAL      = :P_CDFILIAL
                            AND PARASUPRI.NRORG         = :P_NRORG
                            AND (QFOPRODFORN.CDPRODUTO    = PRODUTO.CDPRODUTO)
                            AND (QFOPRODFORN.CDFORNECED   = FORNECEDOR.CDFORNECED)
                            AND ((QFOPRODFORN.IDFORNAPROV = 'N'
                                AND QFOPRODFORN.IDFORNAUTOR   = 'N')
                                OR ((QFOPRODFORN.IDFORNAPROV  = 'S')
                                    AND ((PARASUPRI.IDULTIDTVALQ  = 'M'
                                        AND :P_DTCOL                  > LAST_DAY(QFOPRODFORN.DTVALPRFORN))
                                        OR (PARASUPRI.IDULTIDTVALQ    = 'D'
                                        AND :P_DTCOL                  > QFOPRODFORN.DTVALPRFORN)
                                        )
                                    )
                                )
                        )";
        $params = array(
            'P_CDPRODINI' => $cdprodini,
            'P_CDPRODFIN' => $cdprodfin,
            'P_TPRODUTO' => $tproduto,
            'P_CDFORNECED' => $cdforneced,
            'P_TFORN' => $tforn,
            'P_CDFILIAL' => $cdfilial,
            'P_DTCOL' => $dtcol,
            'P_NRORG' => $nrorg
        );

        try {
            return self::getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica Região por Fornecedor
     *
     * @param String $nrorg      Organização
     * @param String $cdforneced Fornecedor
     * @param String $tforn      Todos Fornecedores
     * @param String $cdregiao   Região
     * @param String $tregiao    Todas Regiões
     *
     * @throws \Exception
     * @return array
     */
    public function verificaRegiaoFornecedor(
        $nrorg, $cdforneced, $tforn, $cdregiao, $tregiao
    ) {
        $sql = "SELECT
                  CDFORNECED
                FROM
                  FORNECEDOR,
                  REGIAO
                WHERE
                 (INSTR(:P_CDFORNECED, CDFORNECED)<> 0 OR :P_TFORN = 'T') AND
                 (INSTR(:P_CDREGIAO, CDREGIAO)<>0 OR :P_TREGIAO = 'T') AND
                 NOT EXISTS
                  (SELECT
                     CDREGIAO
                   FROM
                     REFOREG
                   WHERE
                      REFOREG.CDFORNECED = FORNECEDOR.CDFORNECED AND
                      REFOREG.CDREGIAO = REGIAO.CDREGIAO AND
                      FORNECEDOR.NRORG = :P_NRORG)";

        $params = array(
            'P_CDFORNECED' => $cdforneced,
            'P_TFORN' => $tforn,
            'P_CDREGIAO' => $cdregiao,
            'P_TREGIAO' => $tregiao,
            'P_NRORG' => $nrorg
        );

        try {
            return self::getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna os fornecedores do item de cotação
     *
     * @param String $nrorg       Organização
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da coleta de preço
     * @param String $cdregiao    Região
     *
     * @throws \Exception
     * @return array
     */
    public function getFornecedoresItemCole(
        $nrorg, $cdfilial, $nrcolepreco, $cdregiao
    ) {
        try {
            $sql = "SELECT DISTINCT CDFORNECED
                      FROM ITEMCOLE
                     WHERE ITEMCOLE.NRORG     = :P_NRORG
                       AND ITEMCOLE.CDFILIAL    = :P_CDFILIAL
                       AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO
                       AND ITEMCOLE.CDREGIAO    = :P_CDREGIAO
                    UNION
                    SELECT DISTINCT ITEMCOLE.CDFORNECED
                      FROM ITEMCOLE
                     WHERE ITEMCOLE.NRORG     = :P_NRORG
                       AND ITEMCOLE.CDFILIAL    = :P_CDFILIAL
                       AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO
                       AND ITEMCOLE.CDREGIAO    <> :P_CDREGIAO";

            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDFILIAL' => $cdfilial,
                'P_NRCOLEPRECO' => $nrcolepreco,
                'P_CDREGIAO' => $cdregiao
            );

            $retorno = self::getEntityManager()->getConnection()->fetchAll(
                $sql, $params
            );
            return $retorno;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaFornecedores($nrorg) {

        $stmt = $this->getEntityManager()->getConnection()->prepare("
            SELECT NMFANTFORN,
              NMRAZSOCFORN,
              NRINSJURFORN,
              IDTPIJURFORN,
              CDFORNECED,
              IDSITUCADA,
              IDTIPOFORN,
              NRORG
            FROM FORNECEDOR
            WHERE NRORG = :NRORG
            ORDER BY NRINSJURFORN
        ");

        $stmt->execute(
            array(
                "NRORG" => $nrorg
            )
        );

        return $stmt->fetchAll();

    }
}